@@ -4,7 +4,7 @@ import types
 import pytest
 
 from pluggy import (PluginManager, varnames, PluginValidationError,
-                    HookimplMarker, HookspecMarker)
+                    HookCallError, HookimplMarker, HookspecMarker)
 
 from pluggy import (_MultiCall, _TagTracer, HookImpl, _formatdef)
 
@@ -56,6 +56,15 @@ class TestPluginManager:
         assert len(l) == 1
         assert l == [("hello", a2)]
 
+    def test_has_plugin(self, pm):
+        class A:
+            pass
+
+        a1 = A()
+        pm.register(a1, 'hello')
+        assert pm.is_registered(a1)
+        assert pm.has_plugin('hello')
+
     def test_register_dynamic_attr(self, he_pm):
         class A:
             def __getattr__(self, name):
@@ -235,6 +244,22 @@ class TestPluginManager:
         l = pm.hook.he_method1.call_extra([he_method1], dict(arg=1))
         assert l == [10]
 
+    def test_call_with_too_few_args(self, pm):
+        class Hooks:
+            @hookspec
+            def he_method1(self, arg):
+                pass
+
+        pm.add_hookspecs(Hooks)
+
+        class Plugin1:
+            @hookimpl
+            def he_method1(self, arg):
+                0 / 0
+        pm.register(Plugin1())
+        with pytest.raises(HookCallError):
+            pm.hook.he_method1()
+
     def test_subset_hook_caller(self, pm):
         class Hooks:
             @hookspec
@@ -629,6 +654,26 @@ def test_parse_hookimpl_override():
     assert pm.hook.x1meth2._wrappers[0].hookwrapper
 
 
+def test_plugin_getattr_raises_errors():
+    """Pluggy must be able to handle plugins which raise weird exceptions
+    when getattr() gets called (#11).
+    """
+    class DontTouchMe:
+        def __getattr__(self, x):
+            raise Exception('cant touch me')
+
+    class Module:
+        pass
+
+    module = Module()
+    module.x = DontTouchMe()
+
+    pm = PluginManager(hookspec.project_name)
+    # register() would raise an error
+    pm.register(module, 'donttouch')
+    assert pm.get_plugin('donttouch') is module
+
+
 def test_varnames():
     def f(x):
         i = 3  # noqa
@@ -756,7 +801,7 @@ class Test_MultiCall:
         def f(x):
             return x
         multicall = self.MC([f], {})
-        pytest.raises(KeyError, multicall.execute)
+        pytest.raises(HookCallError, multicall.execute)
 
     def test_call_subexecute(self):
         @hookimpl
@@ -868,7 +913,7 @@ class Test_MultiCall:
 
 
 class TestHookRelay:
-    def test_hapmypath(self, pm):
+    def test_happypath(self, pm):
         class Api:
             @hookspec
             def hello(self, arg):
